c----------------------------------------------------------------------
      program CALTAPM
c----------------------------------------------------------------------
c --- CALTAPM - Convert ASCII TAPM output to a 3D.DAT file format
c----------------------------------------------------------------------
c 
c --- CALTAPM   Version: 1.11     Level: 101129                    MAIN
c
c     Copyright (c) 2010 by Exponent, Inc.
C
c --- Written by: Francoise Robe, Jenny Barclay, and Joseph Scire
c                 TRC
c
c --- UPDATE
c --- V1.1 (101129) to V1.11 (110225) (DGS)
c     1) Updated CALUTILS.FOR to version v2.58 (110225)
c        - Add control file variable type 5 (character array) and
c          retain commas in string returned so that array of
c          variable values can be parsed
c        Modified: READIN, ALTONU, SETVAR
c
c ---------------------------------------------------------------------
c
c --- include parameter file
      include "params.tpm"
c
      character*8 codever,codelevel,dataver,datalevel
      character*16 name3d
      character*64 datamod
      character*132 commenthd 

      character*8  rtime
      character*10 rdate
      character*132 cfinp,lstout,tapmasc,out3d,tapmtop,tapmdef
c
c --- 2-D variables
      integer iland(mxnx,mxny)
      real zs(mxnx,mxny)
      real tsr(mxnx,mxny),radnet(mxnx,mxny),sens(mxnx,mxny)
      real evap(mxnx,mxny),ustar(mxnx,mxny),pvstar(mxnx,mxny)
      real ptstar(mxnx,mxny),wstar(mxnx,mxny),zmix(mxnx,mxny)
      real tscr(mxnx,mxny),rhscr(mxnx,mxny),tsurf(mxnx,mxny)
      real rain(mxnx,mxny), rdum(mxnx,mxny)
      real x(mxnx,mxny),y(mxnx,mxny)
      real rlon(mxnx,mxny),rlat(mxnx,mxny)

c --- 3-D variables
      real z(mxnx,mxny,mxnz)
      real ws(mxnx,mxny,mxnz),wd(mxnx,mxny,mxnz),ww(mxnx,mxny,mxnz)
      real tt(mxnx,mxny,mxnz),rh(mxnx,mxny,mxnz),pt(mxnx,mxny,mxnz)
      real tke(mxnx,mxny,mxnz)
c
c --- For coordinate transformations in COORLDIB
      character*8 cmapi,cmapo
      character*12 caction
      character*4 c4hem
      real*8 vecti(9),vecto(9)

      logical lsohem

c --- Initialize variables
      data cfinp/'caltapm.inp'/,lstout/'caltapm.lst'/
      data tapmasc/'tapm.outa'/,out3d/'3d.dat'/
      data tapmtop/'tapm.top'/,tapmdef/'tapm.def'/

c --- No snow (could change)
      data isnow/0/

c --- Southern Hem
      data lsohem/.TRUE./

c --- dummy variable
      data idum/0/

c --- Set CALTAPM code version and level number
      codever='1.11'
      codelevel='110225'
c
C     Specify 3D.DAT format Version and level number  
      dataver='2.12'    
      datalevel='080102'

c --- Specify output data set names from this code
      name3d='3D.DAT'

      datamod='Header Structure with Comment Lines'
      ncomm=1
      write(commenthd,111)codever,codelevel
 111  format('Produced by CALTAPM Version: ',a8,', Level: ',a8)


c --- Get date and time from system
      call datetm(rdate,rtime,rcpu)
c
c --- Get the name of the control file from the command line
      call comline(cfinp)
c
c --- Open the control file
      open(io5,file=cfinp,status='old')
c
c --- Read control file inputs
c --- Filenames:
      read(5,22)tapmasc
      read(5,22)tapmdef
      read(5,22)tapmtop
      read(5,22)lstout
      read(5,22)out3d

c --- Print filenames:
      write(io6,*) 
      write(io6,*)'Processing the following files:'
      write(io6,*)'TAPM.OUTA filename (input):',tapmasc
      write(io6,*)'TAPM.DEF  filename (input):',tapmdef
      write(io6,*)'TAPM.TOP  filename (input):',tapmtop
      write(io6,*)'CALTAPM M3D filename (output):',out3d
      write(io6,*)
22    format(a132)

c --- Beginning processing date (LST time):
      read(5,45)ibyr,ibmo,ibda,ibhr
      read(5,45)ieyr,iemo,ieda,iehr
45    format(i4,3i2)
c

c --- Open other files
c --- List file 
      open(io6,file=lstout,status='unknown')

c --- TAPM.outa file
      open(io10,file=tapmasc,status='old')

c --- TAPM.TOP file
      open(io20,file=tapmtop,status='old')

c --- TAPM.Def file
      open(io30,file=tapmdef,status='old')

c --- Open output 3D.DAT file
      open(io12,file=out3d,status='unknown')
c
c --- Write program version and level number
      write(io6,10)codever,codelevel
10    format(/1x,'CALTAPM - Version: ',a8,' Level: ',a8//)
c
c #########################################################
c --- Read header records from TAPM.OUTA file
c #########################################################
c --- TAPM gridpoint numbers and horizontal resolution (in m)
      read(io10,*)nx,ny,nz,dx,dy
      write(io6,*)'NX = ',nx,' NY = ',ny,' NZ = ',NZ
      write(io6,*)'DX = ',dx,' DY = ',DY
c --- QA checks
      if(nx.gt.mxnx.or.ny.gt.mxny.or.nz.gt.mxnz)then
         write(io6,*)'Error in CALTAPM'
         write(io6,*)'Number of grid cells (NX, NY, NZ) exceeds ',
     1   'maximum dimensions in model code (MXNX, MXNY, MXNZ)'
         write(io6,*)'NX = ',nx,' MXNX = ',mxnx
         write(io6,*)'NY = ',ny,' MXNY = ',mxny
         write(io6,*)'NZ = ',nz,' MZNZ = ',mxnz
         write(io6,*)'Increase MXNX, MXNY, MXNZ in parameter file ',
     1   '(PARAMS.TPM) and recompile the CALTAPM code'
         stop
      endif
c
c --- Header records - smoothed terrain heights (elevations above msl)
      read(io10,*)((zs(ii,jj),ii=1,nx),jj=1,ny)
      write(io6,*)'ZS(1,1) = ',zs(1,1),' ZS(nx,ny) = ',zs(nx,ny)

c --- Level heights above ground    
      read(io10,*)(((z(ii,jj,kk),ii=1,nx),jj=1,ny),kk=1,nz)
      write(io6,*)'Z(1,1,1) = ',z(1,1,1),' Z(nx,ny,nz) = ',z(nx,ny,nz)


c #############################################################
c --- Compute all gridpoint lat,lon based on origin (TAPM.DEF)
c #############################################################
c --- Skip first 3 records
      do j=1,3
           read(io30,*)
      end do

c --- Lat,lon of grid center
      read(io30,*)rlatc
      read(io30,*)rlonc
      if(rlatc.gt.0) lsohem=.false.
c --- Arbitrary (user-selected) (x,y) of grid center
      read(io30,*)xo
      read(io30,*)yo

c --- Beginning/End Dates of the TAPM run (YYYMMDDHH)
      read(30,44)ibtyr,ibtmo,ibtda 
      read(30,44)ietyr,ietmo,ietda
44    format(i4,2i2)

c --- Begining time always Hour 1 and end Time always hour24 (!! CHECK !!!)
      ibthr=1
      iethr=24

c --- Local Time Zone (real number! LST=GMT+RTZ)
      read(io30,*)rtz
c --- round to nearest hour
      itz=nint(rtz)
      print *,'TIME ZONE:',itz

      write(io6,*)
      write(io6,*)' --- Reading TAPM.DEF file ---'
      write(io6,*)'Lat,lon of grid center',rlatc,rlonc
      write(io6,*)'Beg.date of TAPM run according to TAPM.DEF',
     :             ibtyr,ibtmo,ibtda 
      write(io6,*)'End date of TAPM run according to TAPM.DEF',
     :             ietyr,ietmo,ietda 
      write(io6,*)'Real time zone in TAPM.DEF: ',rtz
      write(io6,*)'Integer time zone in CALTAPM: ',itz
      write(io6,*)' --- Finished reading TAPM.DEF file ---'

c --- Check that processing dates are within simulation dates
      call julday(io6,ibtyr,ibtmo,ibtda,ibtjuldy)
      call julday(io6,ietyr,ietmo,ietda,ietjuldy)
      call julday(io6,ibyr,ibmo,ibda,ibjuldy)
      call julday(io6,ieyr,iemo,ieda,iejuldy)

      call deltt(ibtyr,ibtjuldy,ibthr,ibyr,ibjuldy,
     :           ibhr,jblen)
      if(jblen.lt.0) then
         write(io6,*)'STOP - ERROR'
         write(io6,*)'Processing beginning date before run start'
         write(io6,46)ibtyr,ibtmo,ibtda,ibthr
         write(io6,47)ibyr,ibmo,ibda,ibhr
         print *,'STOP - Check LST file'
      endif

      call deltt(ietyr,ietjuldy,iethr,ieyr,iejuldy,iehr,jblen)

      if(jblen.gt.0) then
         write(io6,*)'STOP - ERROR'
         write(io6,*)'Processing end date after run end'
         write(io6,48)ietyr,ietmo,ietda,iethr
         write(io6,49)ieyr,iemo,ieda,iehr
         print  *, 'STOP - Check LST file'
      endif

c --- Convert to gregorian GMT time for 3D.DAT header output
      ibtjuldyz=ibtjuldy
      ibtyrz=ibtyr
      ibthrz=ibthr

      call incr(io6,ibtyrz,ibtjuldyz,ibthrz,-itz)
      call grday(io,ibtyrz,ibtjuldyz,ibtmoz,ibtdaz)

46    format('Run  starts  (YYYYMMDDHH -LST): ',i4,3i2)
47    format('Proc starts  (YYYYMMDDHH -LST): ',i4,3i2)
48    format('Run ends   (YYYYMMDDHH -LST): ',i4,3i2)
49    format('Proc. ends (YYYYMMDDHH -LST): ',i4,3i2)


c --- Compute TAPM run length 
      call deltt(ibtyr,ibtjuldy,ibthr,ietyr,ietjuldy,iethr,nhours)


c --- Convert center (lat,lon) to UTM-WGS84 (x,y) and get natural UTMzone
c      => xc,yc,iutmo  !! TO DO WITH COORDLIB !!
       call ll2utm(rlatc,rlonc,0,xc,yc,iutmo)   

c --- Grid origin in UTM (m):
      xorgkm=xc-nx*dx/2./1000.
      yorgkm=yc-ny*dy/2./1000.

c --- Compute UTM coordinates of all gridpoints (in km)
      do j=1,ny
         do i=1,nx
            x(i,j)=xorgkm+dx*(i-1)/1000.
            y(i,j)=yorgkm+dy*(j-1)/1000.       
         end do
      end do

      write(6,*)
      write(6,*)'TAPM grid center (lat,lon):',rlatc,rlonc          
      write(6,*)'TAPM resolution dx and dy: ', dx,dy
      write(6,*)'TAPM number of grdpoints nx,ny: ', nx,ny
      write(6,*)'TAPM gridpoint (1,1) in UTM coordinates (km):',
     :          x(1,1), y(1,1)
      write(6,*)'TAPM gridpoint(nx,ny)in UTM coordinates (km): ',
     :          x(nx,ny), y(nx,ny)
      write(6,*)

c --- Convert to gridpoint Latitudes and longitudes 
c      => rlat(i,j),rlon(i,j)   !! TO DO  with COORDLIB!!!
      do i=1,nx
      do j=1,ny
        call utm2ll(x(i,j),y(i,j),iutmo,lsohem,rlat(i,j),rlon(i,j)) 
      end do
      end do
c --- 4 corners LL coordinates
      rxmin=rlat(1,1)
      rxmax=rlat(nx,ny) 
      rymin=rlon(1,1)
      rymax=rlon(nx,ny) 

c #####################################
c --- Read TAPM landuse from TAPM.TOP
c #####################################

c --- Skip first record (nx,ny)
       read(io20,*)nxtop,nytop
c --- Check that that nx,ny in TOP file are consistent with nx,ny in DEF file
c ---      if ((nxtop.ne.nx).or.(nytop.ne.ny)) then
c ---         write(io6,*)'(nx,ny) in top file do not match (nx,ny) in def file"'
c ---         write(io6,*)'TOP file (nx,ny)=' , nxtop,nytop
c ---         write(io6,*)'DEF file (nx,ny)=' , nx,ny
c ---         write(io6,*)'STOP'
c ---         STOP
c ---      endif    
c --- Skip first 2 fields
      read(io20,*)((rdum(ii,jj),ii=1,nx),jj=1,ny)
      read(io20,*)((rdum(ii,jj),ii=1,nx),jj=1,ny)
      read(io20,*)((iland(ii,jj),ii=1,nx),jj=1,ny)
      

c #################################################
c --- Write 3D.DAT header records
c #################################################

c --- Header Record #1
      write(io12,101)name3d,dataver,datamod
 101  format(2(a16),a64)

c --- Header Record #2
      write(io12,'(i4)')ncomm
      do i=1,ncomm
         write(io12,'(a132)')commenthd
      enddo

c --- Set output variable flags for 3D.DAT (vertical velocity and RH/q only)
      ioutw=1
      ioutq=1
      ioutc=0
      iouti=0
      ioutg=0
      iosrf=0

c --- header record #3 
      write (io12,54) ioutw, ioutq, ioutc, iouti, ioutg, iosrf
 54   format (6(i3))

c --- Header record #4 (only dx given but dy could be different)   
c --- set truelat1=truelat2=natural UTM zone
      truelat1=real(iutmo)
      truelat2=truelat1
      write(io12,105)rlatc,rlonc,truelat1,truelat2,
     &      xorgkm,yorgkm,dx/1000.,nx,ny,nz      
 105  format('UTM ',f9.4,f10.4,2f7.2,2f10.3,f8.3,2i4,i3)

c --- Header record #5 (Model physical options - unknown for TAPM - set to 0)
       write(io12,55)idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     : idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum
  55   format (30(i3))

c --- header record #6: 
      write(io12,96)ibtyrz,ibtmoz,ibtdaz,ibthrz,nhours,nx,ny,nz
                    
c --- header record #7:
c --- same extent as in outa file at this point (no subdomain extraction)
      nx1=1
      nx2=nx
      ny1=1
      ny2=ny
      nz1=1
      nz2=nz
      write(io12,97)nx1,ny1,nx2,ny2,nz1,nz2,rxmin,rxmax,rymin,rymax

c --- next nz records:sigma levels 
c --- No sigma level in TAPM. Could be approximated from p(z) 
c --- but not done at the moment. p(z) is a function of time
c --- so sigma levels would be too. Better not to specify them
      do k=nz,1,-1
         write(io12,98)0.
      end do

c --- geophysical records. 
c --- note: lat/lon,elev,land use: all in gridpoint centers
      do 1 j=ny1,ny2
         do 1 i=nx1,nx2
            write(io12,99)i,j,rlat(i,j),rlon(i,j),nint(zs(i,j))
     &          ,iland(i,j)
 1    continue

 96   format(i4,3i2,i5,4i4)
 97   format(6i4,2f10.4,2f9.4)
 98   format(f6.3)
 99   format(2i4,f9.4,f10.4,i5,i3,1x,f9.4,f10.4,i5)


c ###########################################################################
c --- Loop over hours
c
      do 1000 ihr=1,nhours


c --- Read hourly TAPM data
c --- Date & hour (idate: yyyymmdd - hour: 1-24)
      read(io10,50,end=9999)iryr,irmo,irdy,irhr
50    format(i4,2i2,6x,i2)

      write(io6,*)
      write(io6,65)iryr,irmo,irdy,irhr
65    format('Processing YYYY MM DD HH (LST): ',i5,3i3)

      call julday(io6,iryr,irmo,irdy,irjuldy)

c --- Check that beginning time correspond to beg.time in def file
      if (ihr.eq.1) then 
         call deltt(ibtyr,ibtjuldy,ibthr,iryr,irjuldy,irhr,jlen)
         if (jlen.ne.0) then
            write(io6,*)
            write(io6,*)' --- ERROR ---STOP ---'
            write(io6,*)'TAPM times in OUTA and DEF files differ'
            write(io6,*)'Beg. time in OUTA file:',iryr,irmo,irdy,irhr
            write(io6,*)'Beg. time in DEF file: ',
     :                   ibtyr,ibtmo,ibtda,ibthr
            print *,'STOP - CHECK LST FILE'
            print *,'Beginning TAPM times in OUTA and DEF files differ'
            STOP
         endif
      endif     

c --- Exit loop if end processing time is reached
      call deltt(ieyr,iejuldy,iehr,iryr,irjuldy,irhr,jlen)
      if (jlen.gt.0) goto 2000

c --- Convert to gregorian GMT time for 3D.DAT output (output hour: 0-23)
      irjuldyz=irjuldy
      iryrz=iryr 
      irhrz=irhr
      call incr(io6,iryrz,irjuldyz,irhrz,-itz)
      call grday(io,iryrz,irjuldyz,irmoz,irdyz)
      print *,'processing (LST):',iryr,irmo,irdy,irhr,
     :                  ' -- (GMT):',iryrz,irmoz,irdyz,irhrz

c --- Read 2-D arrays
c --- Total solar radiation (W/m2)
      read(io10,*,end=9999)((tsr(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'TSR(1,1) = ',tsr(1,1),' TSR(nx,ny) = ',tsr(nx,ny)
c --- Net radiation (W/m2)
      read(io10,*,end=9999)((radnet(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'RADNET(1,1) = ',radnet(1,1),' RADNET(nx,ny) = ',
c     1 radnet(nx,ny)
c --- Sensible heat flux (W/m2)
      read(io10,*,end=9999)((sens(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'SENS(1,1) = ',sens(1,1),' SENS(nx,ny) = ',
c     1 sens(nx,ny)
c --- Evaporative heat flux (W/m2)
      read(io10,*,end=9999)((evap(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'EVAP(1,1) = ',evap(1,1),' EVAP(nx,ny) = ',
c     1 evap(nx,ny)
c --- Friction veloctiy scale (m/s)
      read(io10,*,end=9999)((ustar(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'USTAR(1,1) = ',ustar(1,1),' USTAR(nx,ny) = ',
c     1 ustar(nx,ny)
c --- Potential virtual temperature scale (K)
      read(io10,*,end=9999)((pvstar(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'PVSTAR(1,1) = ',pvstar(1,1),' PVSTAR(nx,ny) = ',
c     1 pvstar(nx,ny)
c --- Potential temperature scale (K)
      read(io10,*,end=9999)((ptstar(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'PTSTAR(1,1) = ',ptstar(1,1),' PTSTAR(nx,ny) = ',
c     1 ptstar(nx,ny)
c --- Vertical veloticy scale (m/s)
      read(io10,*,end=9999)((wstar(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'WSTAR(1,1) = ',wstar(1,1),' WSTAR(nx,ny) = ',
c     1 wstar(nx,ny)
c --- Mixing Height (m)
      read(io10,*,end=9999)((zmix(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'ZMIX(1,1) = ',zmix(1,1),' ZMIX(nx,ny) = ',
c    1 zmix(nx,ny)
c --- Screen-level temperature  (K)
      read(io10,*,end=9999)((tscr(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'TSCR(1,1) = ',tscr(1,1),' TSCR(nx,ny) = ',
c     1 tscr(nx,ny)
c --- Screen-level rel.hum (%)
      read(io10,*,end=9999)((rhscr(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'RHSCR(1,1) = ',rhscr(1,1),' RHSCR(nx,ny) = ',
c     1 rhscr(nx,ny)
c --- Surface temperature scale (K)
      read(io10,*,end=9999)((tsurf(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'TSURF(1,1) = ',tsurf(1,1),' TSURF(nx,ny) = ',
c     1 tsurf(nx,ny)
c --- Rainfall(mm/hr)
      read(io10,*,end=9999)((rain(ii,jj),ii=1,nx),jj=1,ny)
c      write(io6,*)'RAIN(1,1) = ',rain(1,1),' RAIN(nx,ny) = ',
c     1 rain(nx,ny)

c --- Read 3-D variables
c --- Horizontal wind speed (m/s)
      read(io10,*,end=9999)
     :  (((ws(ii,jj,kk),ii=1,nx),jj=1,ny),kk=1,nz)
c      write(io6,*)'WS = ',ws(1,1,1),ws(nx,ny,nz)
c --- Horizontal wind direction (m/s)
      read(io10,*,end=9999)
     :   (((wd(ii,jj,kk),ii=1,nx),jj=1,ny),kk=1,nz)
c      write(io6,*)'WD = ',wd(1,1,1),wd(nx,ny,nz)
c --- Vertical velocity (m/s)
      read(io10,*,end=9999)
     :   (((ww(ii,jj,kk),ii=1,nx),jj=1,ny),kk=1,nz)
c      write(io6,*)'WW = ',ww(1,1,1),ww(nx,ny,nz)
c --- Temperature (K)
      read(io10,*,end=9999)
     :   (((tt(ii,jj,kk),ii=1,nx),jj=1,ny),kk=1,nz)
c      write(io6,*)'TT = ',tt(1,1,1),tt(nx,ny,nz)
c --- Relative Humidity (%)
      read(io10,*,end=9999)
     :   (((rh(ii,jj,kk),ii=1,nx),jj=1,ny),kk=1,nz)
c      write(io6,*)'RH = ',rh(1,1,1),rh(nx,ny,nz)
c --- Potential Temperature (K)
      read(io10,*,end=9999)
     :   (((pt(ii,jj,kk),ii=1,nx),jj=1,ny),kk=1,nz)
c      write(io6,*)'PT = ',pt(1,1,1),pt(nx,ny,nz)
c --- Turbulent kinetic energy(m2/s2)
      read(io10,*,end=9999)
     :   (((tke(ii,jj,kk),ii=1,nx),jj=1,ny),kk=1,nz)
c      write(io6,*)'TKE = ',tke(1,1,1),tke(nx,ny,nz)
c
c ########################################
c --- Write hourly data in 3D.DAT format
c ########################################

c ---    loop over i,j
         do 100 j=ny1,ny2
         do 100 i=nx1,nx2

c ---       Rain accumulated in 1hr in cm:
            prec= rain(i,j)/10.
c
c ---       Compute sf pressure from T(k=1) and Theta(k=1) (mb)
            ps=1000*(tt(i,j,1)/pt(i,j,1))**(1./0.286)

c ---       Saturation vapor pressure at Tscreen (Bolton formula -mb)
            tc=tscr(i,j)-273.15
            es=6.112*exp(17.67*tc/(tc+243.5))
c ---       Saturation mixing ratio
            wqs=0.622*es/(ps-es)
c ---       Compute sf vapor mixing ratio (g/kg)
            qscr=wqs*rhscr(i,j)*10.

c ---       Assume short wave radiation flux is total solar flux
c ---       Long wave radiation flux is total rad-solar
            rlwd=radnet(i,j)-tsr(i,j)

c ---       Loop over vertical levels (from surface to top)
            do k=nz1,nz2

c ---          Compute height above msl (m)
               zmsl=zs(i,j)+z(i,j,k)

c ---          Compute pressure from T and Theta (mb)
               ps=1000*(tt(i,j,k)/pt(i,j,k))**(1./0.286)

c ---          Saturation vapor pressure (Bolton formula -mb)
               tc=tt(i,j,k)-273.15
               es=6.112*exp(17.67*tc/(tc+243.5))

c ---          Saturation mixing ratio
               wqs=0.622*es/(ps-es)

c ---          Compute vapor mixing ratio (g/kg)
               qv=wqs*rh(i,j,k)*10.

c ---          Write time and surface record
               if(k.eq.nz1) then
                  write(io12,91)iryrz,irmoz,irdyz,irhrz,
     :              i,j,ps,prec,isnow,tsr(i,j),rlwd,
     :              tscr(i,j),qscr,wd(i,j,k),ws(i,j,k),tscr(i,j)
              
               endif
 91            format(i4,3i2,2i3,f7.1,f5.2,i2,3f8.1,f8.2,3f8.1)
               
c ---          Write 3D.DAT record
               write(io12,93)int(ps),int(zmsl),tt(i,j,k),
     :           int(wd(i,j,k)),ws(i,j,k),ww(i,j,k),
     :           int(rh(i,j,k)),qv
 93            format(i4,i6,f6.1,i4,f5.1,f6.2,i3,f5.2)
            end do

100     continue


1000  continue
c
2000  continue
      write(io6,*)
      write(io6,*)' --- CALTAPM Successfully Completed ---'

      print*, '--- CALTAPM Successfully Completed ---'
      stop

c --- Read past end of OUTA records
9999  write(io6,*) '---ERROR---'
      write(io6,*)'Read past end of file - Check for missing records'
      print *, ' Read past end of file - Check for missing records'

      end
c----------------------------------------------------------------------
c --- Include CALUTILS utility routines
      include "calutils.for"
c --- Include COORDLIB Coordinate transformation routines
c     include "coordlib.for"
c----------------------------------------------------------------------
      subroutine ll2utm(rlat,rlon,iz0,x,y,iz)                                  
c----------------------------------------------------------------------        
c
c --- CALTAPM   Version: 1.11     Level: 101129                  LL2UTM
c
c --- Routine from:                                                                             
c --- CALMET   Version: 5.2       Level: 991104a                 LL2UTM        
c                                                                              
c ---          Reset dl to range -180:180 degrees, for zone 1/60 overlap       
c              Modification B. de Foy (7/99)                                   
c                                                                              
c --- PURPOSE:  Converts latitude/longitude to UTM coordinates                 
c                                                                              
c           *** Universal Transverse Mercator (UTM) grid system divides        
c           *** the world into 60 north-south zones, covering a 6 deg.         
c           *** strip of longitude. Zone 1 begins between 180 and 174          
c           *** degrees West longitude and progresses eastward to              
c           *** zone 60.                                                       
c           *** This routine works in both No. & So. Hemispheres               
c               Reference --                                                   
c                 "Map Projections--A Working Manual", p61,                    
c                  U.S. Geological Survey Professional Paper 1395,             
c                    Note: assumes the Clarke 1866 ellipsoid                   
c               Adapted from --                                                
c                  EPS version 2.0; subr. MAPGTU                               
c                                                                              
c --- INPUTS:                                                                  
c               RLAT - Real        - N Latitude in decimal degrees             
c                                    (use negative for southern hemisphere)    
c               RLON - Real        - E Longitude in decimal degrees            
c                                    (use negative for western hemisphere)     
c                IZ0 - Integer     - UTM zone override (used only if           
c                                    IZ0 .ne. zero).                           
c                                                                              
c --- OUTPUT:                                                                  
c                  X - Real        - UTM easting in km                         
c                  Y - Real        - UTM northing in km                        
c                 IZ - Integer     - UTM zone                                  
c                                                                              
c --- LL2UTM called by:  MAIN                                                
c --- LL2UTM calls:      none                                                  
c----------------------------------------------------------------------        
                                                                               
      real k0                                                                  
      real N,M                                                                 
                                                                               
      parameter (k0=0.9996)                                                    
      parameter (a=6378206.4)                                                  
      parameter (e2=0.00676866)                                                
      parameter (ep2=0.0068148)                                                
      parameter (false_e=500000.0)                                             
      parameter (pi=3.141592654)                                               
      parameter (dtr=3.141592654/180.0)                                        
                                                                               
      if (iz0 .eq. 0) then                                                     
c ---   Locate natural zone                                                    
          iz = int((180.0+rlon)/6.0) + 1                                       
      else                                                                     
c ---   Zone override                                                          
          iz = iz0                                                             
      endif                                                                    
                                                                               
c --- Compute delta longitude in radians                                       
      dl = dtr*(rlon - (6.0*iz-183.0))                                         
c --- Reset to range between -pi and pi radians - bdf                          
      if (dl.lt.(-1.0*pi)) dl = dl + 2.0*pi                                    
      if (dl.gt.pi)        dl = dl - 2.0*pi                                    
                                                                               
c --- Convert phi (latitude) to radians                                        
      p = dtr*rlat                                                             
                                                                               
      sinp = sin(p)                                                            
      N = a/sqrt(1.0-e2*sinp*sinp)                                             
      tanp = tan(p)                                                            
      T = tanp*tanp                                                            
      cosp = cos(p)                                                            
      C = ep2*cosp*cosp                                                        
      A1 = dl*cosp                                                             
      M = 111132.0894*rlat - 16216.94*sin(2.0*p) + 17.21*sin(4.0*p)            
     &  - 0.02*sin(6.0*p)                                                      
                                                                               
      A2 = A1**2                                                               
      A3 = A2*A1                                                               
      A4 = A2**2                                                               
      A5 = A4*A1                                                               
      A6 = A4*A2                                                               
      T2 = T**2                                                                
                                                                               
c --- Compute UTM x and y (km)                                                 
      x = 0.001*(k0*N*(A1+(1.0-T+C)*A3/6.0                                     
     &  + (5.0-18.0*T+T2+72.0*C-58.0*ep2)*A5/120.0)                            
     &  + false_e)                                                             
      y = (M+N*tanp * (A2/2.0 + (5.0-T+9.0*C+4.0*C*C)*A4/24.0                  
     &  + (61.0-58.0*T+T2+600.0*C-330.0*ep2)*A6/720.0))                        
      false_n = 0.                                                             
      if (rlat .lt. 0.) then                                                   
c --- in km, unlike false_e                                                    
        false_n = 10000.                                                       
      endif                                                                    
      y = 0.001*k0*y + false_n                                                 
                                                                               
      return                                                                   
      end                                                                      
c----------------------------------------------------------------------        
      subroutine utm2ll(x,y,iz,lsohem,rlat,rlon)                               
c----------------------------------------------------------------------        
c 
c --- CALTAPM   Version: 1.11     Level: 101129                  UTM2LL
c
c --- Routine from:                                                                                                                                                          
c --- CALMET   Version: 5.2       Level: 991104a                 UTM2LL        
c                                                                              
c --- Reset rlong to range -180:180 degrees, for zone 1/60 overlap             
c     Modification B. de Foy (7/99)                                            
c                                                                              
c --- PURPOSE:  Converts UTM coordinates to latitude/longitude                 
c               Works in both Northern & Southern Hemispheres                  
c               Reference--                                                    
c                 "Map Projections--A Working Manual", p61,                    
c                  U.S. Geological Survey Professional Paper 1395,             
c                    Note: assumes the Clarke 1866 ellipsoid                   
c               Adapted from --                                                
c                  EPS version 2.0; subr. MAPUTG                               
c                                                                              
c --- INPUTS:                                                                  
c                  X - real    - UTM easting in km                             
c                  Y - real    - UTM northing in km                            
c                 IZ - integer - UTM zone (6 deg N-S strip, range=1,60)        
c             LSOHEM - logical - TRUE = southern hemisphere                    
c                                FALSE = northern hemisphere                   
c                                                                              
c --- OUTPUT:                                                                  
c               RLAT - real    - N Latitude in decimal degrees                 
c               RLON - real    - E Longitude in decimal degrees                
c                                                                              
c --- UTM2LL called by:  MAIN                                                
c --- UTM2LL calls:      none                                                  
c----------------------------------------------------------------------        
                                                                               
      real k0,M,N1,l                                                           
      logical lsohem                                                           
                                                                               
      parameter (k0=0.9996)                                                    
      parameter (a=6378206.4)                                                  
      parameter (e1=0.001697916)                                               
      parameter (e11=3.0*e1/2.0 - 27.0*e1*e1*e1/32.0)                          
      parameter (e12=21.0*e1*e1/16.0 - 55.0*e1*e1*e1*e1/32.0)                  
      parameter (e13=151.0*e1*e1*e1/96.0)                                      
      parameter (e14=1097.0*e1*e1*e1*e1/512.0)                                 
      parameter (e2=0.00676866)                                                
      parameter (e4=e2*e2)                                                     
      parameter (e6=e2*e4)                                                     
      parameter (ep2=0.0068148)                                                
      parameter (false_e=500000.0)                                             
      parameter (rtd=180.0/3.141592654)                                        
                                                                               
c --- Parameter definitions                                                    
c      k0        -  scale on central meridian                                  
c      a         -  Clarke 1866 equatorial radius                              
c      e2        -  squared Clarke 1866 eccentricity                           
c      ep2       -  (e2/(1.0-e2)                                               
c      false_e   -  false easting                                              
c      rtd       -  radians to degrees conversion                              
                                                                               
c --- Central meridian                                                         
      rlon0 = iz*6.0 - 183.0                                                   
                                                                               
c --- Correct for false easting, southern hemisphere and change to meters      
      xm = 1000.0*x - false_e                                                  
      if(LSOHEM) then                                                          
        ym = 1000.0 * (y-10000.)                                               
      else                                                                     
        ym = 1000.0 * y                                                        
      endif                                                                    
                                                                               
      M = ym/k0                                                                
      u = M/(a*(1.0-e2/4.0 - 3.0*e4/64.0 - 5.0*e6/256.0))                      
      p1 = u + e11*sin(2.0*u) + e12*sin(4.0*u) + e13*sin(6.0*u) +              
     1         e14*sin(8.0*u)                                                  
      cosp1 = cos(p1)                                                          
      C1 = ep2*cosp1**2                                                        
      C2 = C1**2                                                               
      tanp1 = tan(p1)                                                          
      T1 = tanp1**2                                                            
      T2 = T1**2                                                               
      sinp1 = sin(p1)                                                          
      sin2p1 = sinp1**2                                                        
      N1 = a/sqrt(1.0-e2*sin2p1)                                               
      R0 = 1.0-e2*sin2p1                                                       
      R1 = a*(1.0-e2)/sqrt(R0**3)                                              
                                                                               
      D = xm/(N1*k0)                                                           
      D2=D**2                                                                  
      D3=D*D2                                                                  
      D4=D*D3                                                                  
      D5=D*D4                                                                  
      D6=D*D5                                                                  
                                                                               
      p = p1 - (N1*tanp1/R1) * (D2/2.0                                         
     1       - (5.0+3.0*T1+10.0*C1-4.0*C2-9.0*ep2)*D4/24.0                     
     2       + (61.0+90.0*T1+298.0*C1+45.0*T2-252*ep2-3.0*C2)*D6/720.0)        
      rlat = rtd*p                                                             
      l = (D - (1.0+2.0*T1+C1)*D3/6.0                                          
     1       + (5.0-2.0*C1+28*T1-3.0*C2+8.0*ep2+24.0*T2)*D5/120.0)/cosp1       
      rlon = rtd*l + rlon0                                                     
                                                                               
c reset rlon to range -180 to 180 degrees - bdf                                
      if (rlon.lt.-180.0) rlon = rlon + 360.0                                  
      if (rlon.gt.180.0) rlon = rlon - 360.0                                   
                                                                               
      return                                                                   
      end                                                                      
c----------------------------------------------------------------------        

 
